/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.bundu.bezugswst;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.dive4elements.river.artifacts.bundu.BunduResultType;
import org.dive4elements.river.artifacts.common.AbstractCalculationExportableResult;
import org.dive4elements.river.artifacts.common.AbstractExportContext;
import org.dive4elements.river.artifacts.common.ExportContextCSV;
import org.dive4elements.river.artifacts.common.ExportContextPDF;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.MetaAndTableJRDataSource;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;

/**
 * Contains the results of a //TODO LINK{@link BezugswstCalculation}.
 *
 * @author Gernot Belger
 */
final class BezugswstMissVolCalculationResult2 extends AbstractCalculationExportableResult implements BunduWstExporter.MissResultType {

    private static final long serialVersionUID = 1L;

    private static final String JASPER_FILE = "/jasper/templates/bundu.bezugswst.result2.jrxml";
    private static final String fieldMsgStr = "bundu.export.bezugswst.csv.meta.miss.field";
    private static final int fieldSize = 10;

    public BezugswstMissVolCalculationResult2(final String label, final Collection<ResultRow> rows) {
        super(label, rows);

    }

    @Override
    protected void writeCSVResultMetadata(final ExportContextCSV exportContextCSV) {
        // no metadata
    }

    @Override
    protected String getJasperFile() {
        return JASPER_FILE;
    }

    protected String[] formatRow(final AbstractExportContext exportContext, final ResultRow row) {

        final Collection<String> lines = new ArrayList<>(20);

        lines.add(exportContext.formatRowValue(row, GeneralResultType.station));
        final List<String> fields = (List<String>) row.getValue(BunduResultType.fields);
        assert (fields.size() == fieldSize); // immer abgleichen mit der Calculation!
        for (final String field : fields) {
            lines.add(field);
        }
        lines.add(exportContext.formatRowValue(row, BunduResultType.meanBedheight));

        return lines.toArray(new String[lines.size()]);
    }

    @Override
    public void writeCSVHeader(final ExportContextCSV exportContextCSV, final RiverInfo river) {

        final int colSize = fieldSize + 2;
        exportContextCSV.writeTitleForTabs("bundu.export.csv.title.bezugswst.result2", colSize); // Voraussetzung für Tabs ist, dass der Titel vor den Headern
        final Collection<String> header = new ArrayList<>(colSize);

        header.add(exportContextCSV.formatCsvHeader(GeneralResultType.station));

        for (int i = 1; i <= fieldSize; i++) {
            header.add(exportContextCSV.msg(fieldMsgStr, i));
        }
        header.add(exportContextCSV.formatCsvHeader(BunduResultType.meanBedheight));

        exportContextCSV.writeCSVLine(header.toArray(new String[header.size()]));

    }

    @Override
    protected String[] formatCSVRow(final ExportContextCSV exportContextCSV, final ResultRow row) {

        return this.formatRow(exportContextCSV, row);
    }

    @Override
    protected String[] formatPDFRow(final ExportContextPDF exportContextPDF, final ResultRow row) {

        return this.formatRow(exportContextPDF, row);
    }

    @Override
    protected void addJRTableHeader(final ExportContextPDF exportContextPDF, final MetaAndTableJRDataSource source) {
        /* column headings */

        exportContextPDF.addJRMetadata(source, "station_header", GeneralResultType.station);
        for (int i = 1; i <= fieldSize; i++) {
            exportContextPDF.addJRMetadata(source, "field_header" + String.valueOf(i), exportContextPDF.msg(fieldMsgStr, i));
        }
        exportContextPDF.addJRMetadata(source, "mean_bedheight_header", BunduResultType.meanBedheight);

    }
}